@@ -1,66 +1,73 @@
 #
 # $Source: /home/nlfm/Working/Zircon/Released/lib/RCS/Friends.tcl,v $
-# $Date: 1996/07/15 12:55:23 $
-# $Revision: 1.17.1.4 $
+# $Date: 1997/06/09 12:02:53 $
+# $Revision: 1.18.1.23 $
 #
+package provide zircon 1.18
 #
 # Handle all stuff related to Friends menu/window
 #
 #
-class Friend {
-    nick	{}
-    lnick	{}
-    notify	1
-    net		{}
-    usr		nil
-    id		{}
-    menu	1
-    ison	0
-}
-#
 proc Friend_save {desc net} {
+    set cm 1
     foreach x [$net friends] {
-	regsub -all {[][\\{\"}]} [$x name] {\\&} n
-	set ln "Friend $n"
+	set ln [list Friend [$x nick]]
 	if ![$x menu] { append ln " -menu 0"}
 	if ![$x notify] { append ln " -notify 0" }
-	if ![string match {} [$x id]] { append ln " -id {[$x id]}" }
+	switch {} [$x id] {} default {append ln " -id {[$x id]}"}
+	if $cm { puts $desc "#\n# Users\n#" ; set cm 0 }
 	puts $desc $ln
     }
 }
 #
-proc Friend_find {name} {
-    global current
-    upvar #0 FTO$current(net) FTO UTO$current(net) UTO
+proc Friend_find {name net} {
+    upvar #0 FTO$net FTO UTO$net UTO
     set ln [string tolower $name]
     if [info exists FTO($ln)] { return $FTO($ln) }
     if [info exist UTO($ln)] { return [$UTO($ln) fobj] }
     return nil
 }
 #
-proc Friend_make {name} {
-    global current
-    upvar #0 FTO$current(net) FTO
+proc Friend_make {net name} {
+    upvar #0 FTO$net FTO
     set ln [string tolower $name]
     if [info exists FTO($ln)] { return $FTO($ln) }
-    return [Friend $name]
+    return [$net eval [list Friend $name]]
 }
 #
 proc Friend {name args} {
-    if ![string compare :: $name] {
-	return [eval Friend_[lindex $args 0] [lrange $args 1 end]]
-    }
+    switch -- $name :: {return [eval Friend_[lindex $args 0] [lrange $args 1 end]]}
     set this [objName Friend]
     initObj $this Friend
-    global current
-    set net $current(net)
+    upvar #0 currentNet net
     upvar #0 FTO$net FTO $this fdata
     set fdata(nick) [set name [$net trimNick $name]]
     set fdata(net) $net
     $net register friends $this
-    set FTO([set fdata(lnick) [string tolower $name]]) $this
+    set fdata(lnick) [string tolower $name]
     proc $this {args} "eval friend_call $this \$args"
-    if ![string match {} $args] { eval $this configure $args }
+    eval $this configure $args
+    if [info exists FTO($fdata(lnick)] {
+	set other $FTO($fdata(lnick))
+	switch -- $fdata(id) [$other id] {
+	    $this delete
+	    return nil
+	} default {
+	    switch {} $fdata(id) {$this delete ; return nil }
+	    switch {} [$other id] {$other delete ; return nil }
+# OOPS!!! Need to rewrite the friends mechanisms to cope with this
+# one!!
+	    tellError {} Friends {You have two Friend declarations \
+with the same name but different id's. Currently Zircon cannot handle
+this.
+
+Sorry!!}
+	    $this delete
+	    return nil	    
+	}
+    } {
+	set FTO([set fdata(lnick) [string tolower $name]]) $this
+    }
     return $this
 }
 #
@@ -71,13 +78,15 @@ proc Friend_pack {net} {
 #
 proc Friend_unpack {net} {
     upvar #0 ${net}FTO newFTO
-    foreach u [array names newFTO] { $newFTO($u) unpack $net }
-    unset newFTO
+    if [info exists newFTO] {
+	foreach u [array names newFTO] { $newFTO($u) unpack $net }
+	unset newFTO
+    }
 }
 #
 proc friend_doNotify {this} {
     upvar #0 $this fdata
-    if [string compare nil $fdata(usr)] {
+    switch nil $fdata(usr) {} default {
 	$fdata(usr) configure -notify $fdata(notify)
     }
     if $fdata(notify) { [$this net] ISON } {
@@ -92,26 +101,26 @@ proc friend_pack {this net} {
 }
 #
 proc friend_unpack {this net} {
-    upvar #0 new$this newu $this fdata
+    upvar #0 new$this newu ${net}FTO newt $this fdata
     foreach v {nick notify menu id} {$this configure -$v $newu($v)}
-    unset newu
-    $this configure -usr [User :: find [$this nick]]
+    unset newu newt($fdata(lnick))
+    $this configure -usr [User :: find [$this nick] $net]
 }
 #
 proc friend_name {this} {
     upvar #0 $this fdata
-    if [string compare nil $fdata(usr)] { return [$fdata(usr) name] }
-    return $fdata(nick)
+    switch nil $fdata(usr) {return $fdata(nick)}
+    return [$fdata(usr) name]
 }
 #
 proc friend_lname {this} {
     upvar #0 $this fdata
-    if [string compare nil $fdata(usr)] { return [$fdata(usr) lname] }
-    return $fdata(lnick)
+    switch nil $fdata(usr) {return $fdata(lnick)}
+    return [$fdata(usr) lname]
 }
 #
 proc friend_delete {this} {
-    $this configure -notify 0
+    $this configure -notify 0 -usr nil
     upvar #0 $this fdata
     set net $fdata(net)
     upvar #0 FTO$net FTO
@@ -135,7 +144,19 @@ proc friend_configure {this args} {
 		set FTO($fdata(lnick)) $this
 	    }
 	-usr {
-		if [string compare nil $val] { $val configure -fobj $this }
+		switch nil $fdata(usr) {} default {
+		    $fdata(usr) configure -fobj nil
+		}
+		switch nil $val {} default {$val configure -fobj $this}
+		set fdata(usr) $val
+	    }
+	-id {
+		if [catch {regexp -nocase $val abcde}] {
+		    tellError {} {Id Error} "Invalid regular\
+expression in -id option of Friend [$this name]"
+		} {
+		    set fdata(id) $val
+		}
 	    }
 	default { set fdata([string range $op 1 end]) $val }
 	}
@@ -163,15 +184,13 @@ proc friends_rename {this frd nnk} {
     if [$this menu] {
 	global Fctl
 	set ctl [$Fctl($this) window]
-	if {[set x [indexHack $ctl.bf1.friends.menu [$frd name] 0]] >= 0} {
-	    $ctl.bf1.friends.menu entryconfigure $x -label $nnk
+	if {[set x [indexHack $ctl.friends.menu [$frd name] 1]] >= 0} {
+	    $ctl.friends.menu entryconfigure $x -label $nnk
 	}
     } {
-	if [winfo exists .@$this.users.userList.$frd] {
-	    if [string compare $nnk [$frd nick]] {
-		append nnk "\n([$frd nick])"
-	    }
-	    .@$this.users.userList.$frd config -text $nnk
+	if [winfo exists .@$this.userList.$frd] {
+	    switch -- $nnk [$frd nick] {} default {append nnk "\n([$frd nick])"}
+	    .@$this.userList.$frd config -text $nnk
 	}
     }
 }
@@ -180,28 +199,29 @@ proc friends_absent {this frd} {
     if [$this menu] {
 	global Fctl
 	set ctl [$Fctl($this) window]
-	if {[set x [indexHack $ctl.bf1.friends.menu [$frd name] 0]] >= 0} {
-	    set v [$ctl.bf1.friends.menu entrycget $x -state]
-	    return [string compare normal $v]
+	if {[set x [indexHack $ctl.friends.menu [$frd name] 1]] >= 0} {
+	    set v [$ctl.friends.menu entrycget $x -state]
+	    switch normal $v {} default {return 1}
 	}
 	return 0	
     } {
-	return [expr {[winfo exists .@$this.users.userList.$frd] \
-	      && ![normal .@$this.users.userList.$frd]}]
+	return [expr {[winfo exists .@$this.userList.$frd] \
+	      && ![normal .@$this.userList.$frd]}]
     }
 }
 #
 proc friends_menu {this} {
     global Fctl
-    return [string match {menu} [[$Fctl($this) net] friendsStyle]]
+    switch menu [[$Fctl($this) net] friendsStyle] {return 1}
+    return 0
 }
 #
 proc friends_mark {this frd what} {
     if [$this menu] {
 # Dont do anything at the moment
     } \
-    elseif [winfo exists .@$this.users.userList.$frd] {
-	markButton .@$this.users.userList.$frd $what
+    elseif [winfo exists .@$this.userList.$frd] {
+	markButton .@$this.userList.$frd $what
 	popup .@$this
     }
 }
@@ -213,14 +233,14 @@ proc friends_state {this state args} {
 	global Fctl
 	set ctl [$Fctl($this) window]
 	foreach id $args {
-	    if {[set x [indexHack $ctl.bf1.friends.menu $id 0]] >= 0} {
-		$ctl.bf1.friends.menu entryconfigure $x -state $state
+	    if {[set x [indexHack $ctl.friends.menu $id 1]] >= 0} {
+		$ctl.friends.menu entryconfigure $x -state $state
 	    }
 	}
     } {
 	foreach id $args {
-	    if [winfo exists .@$this.users.userList.$id] {
-		.@$this.users.userList.$id conf -state $state
+	    if [winfo exists .@$this.userList.$id] {
+		.@$this.userList.$id conf -state $state
 	    }
 	}
     }
@@ -235,28 +255,26 @@ proc Friends {name args} {
 #
 proc friends_show {this} {
     if [$this menu] {
-	makeFriendsMenu $this [$Fctl($this) window].bf1.friends.menu
+	global Fctl
+	makeFriendsMenu $this [$Fctl($this) window].friends.menu
     } elseif {[winfo exists .@$this]} {popup .@$this} {makeFriends $this}
 }
 #
 proc makeFriendsMenu {this win} {menu $win ; insertFriends $this}
 #
 proc makeFriends {this} {
-    global ztrans
     set w [toplevel .@$this -class Zircon]
     wm title $w "Friends"
     wm resizable $w 0 1
     wm protocol $w WM_DELETE_WINDOW "killWindow $w"
 
-    set win [frame $w.users -relief raised]
-    scrollbar $win.vscroller -command "$win.userList yview" 
-    text $win.userList -yscrollcommand "bsSet $win.vscroller" \
+    scrollbar $w.vscroller -command "$w.userList yview" 
+    text $w.userList -yscrollcommand "$w.vscroller set" \
       -width 14 -height 10 -relief flat -borderwidth 0
-    bindtags $win.userList {null}
-    button $win.ok -text $ztrans(dismiss) -command "killWindow $w"
-    pack $win.ok -side right -fill y
-    pack $win.userList -side left -fill y -expand 1
-    pack $win -expand 1 -fill y
+    bindtags $w.userList {null}
+    button $w.ok -text [trans dismiss] -command "killWindow $w"
+    grid $w.userList $w.vscroller -sticky ns
+    grid $w.ok -sticky ew
     insertFriends $this
 }
 #
@@ -277,17 +295,17 @@ proc insertFriends {this} {
 }
 #
 proc friends_add {this frd} {
-    if [string compare nil $frd] {
+    switch nil $frd {} default {
 	if [$this menu] {
 	    global Fctl
-	    set w [$Fctl($this) window].bf1.friends.menu
+	    set w [$Fctl($this) window].friends.menu
 	    if ![winfo exists $w.$frd] {
 		$w add cascade -label [$frd name] -menu $w.$frd
 		makeUserMenu nil $w.$frd $frd
 	    }
 	} \
 	elseif [winfo exists .@$this] {
-	    set win .@$this.users.userList
+	    set win .@$this.userList
 	    set winb $win.$frd
 	    if ![winfo exists $winb] {
 		menubutton $winb -text [$frd name] -menu $winb.menu -width 12
@@ -300,17 +318,18 @@ proc friends_add {this frd} {
 }
 #
 proc friends_remove {this frd} {
+    switch nil $frd return
     global Fctl
     if [$this menu] {
 	set ctl [$Fctl($this) window]
-	if {[set x [indexHack $ctl.bf1.friends.menu [$frd name] 0]] >= 0} {
-	    $ctl.bf1.friends.menu delete $x
+	if {[set x [indexHack $ctl.friends.menu [$frd name] 1]] >= 0} {
+	    $ctl.friends.menu delete $x
 	}
     } \
-    elseif [winfo exists .@$this.users.userList.$frd] {
+    elseif [winfo exists .@$this.userList.$frd] {
 	if [[$Fctl($this) net] friendsOn] {
-	    if [normal .@$this.users.userList.$frd] {
-		destroy .@$this.users.userList.$frd
+	    if [normal .@$this.userList.$frd] {
+		destroy .@$this.userList.$frd
 	    }
 	} {
 	    $this mark $frd {}
