@@ -1,46 +1,46 @@
 #
 # $Source: /home/nlfm/Working/Zircon/Released/lib/RCS/Notice.tcl,v $
-# $Date: 1996/08/29 11:22:57 $
-# $Revision: 1.17.1.5 $
+# $Date: 1997/05/21 12:55:23 $
+# $Revision: 1.18.1.24 $
 #
-class Notice {
-    buttons	0
-    width	80
-    height	10
-    away	{}
-}
+package provide zircon 1.18
 #
 proc Notice {name args} {
-    if ![string compare {::} $name] {
+    switch -- $name :: {
 	set op [lindex $args 0]
-	if [string match {} [info procs Notice_$op]] {
-	    return [eval Channel_$op [lrange $args 1 end] ]
-	}
+	switch [info procs Notice_$op] {} {return [eval Channel_$op [lrange $args 1 end] ]}
 	return [eval Notice_$op [lrange $args 1 end] ]
     }
-    if ![string compare [set id [Notice :: find $name]] nil] {set id [makeNotice $name]}
-    if ![string match {} $args] { eval $id configure $args }
+    global currentNet
+    switch nil [set id [Notice :: find $name $currentNet]] {
+	set id [makeNotice $name $currentNet]
+    }
+    eval $id configure $args
     return $id
 }
 #
 proc notice_call {this op args} {
-    if [string match {} [info procs notice_$op]] {
+    switch {} [info procs notice_$op] {
 	return [eval channel_call $this $op $args]
     }
     return [eval notice_$op $this $args]
 }
 #
-proc makeNotice {chan} {
-    global defNotice defChan current
+proc makeNotice {chan net} {
+    global defNotice
     set this [objName Notice]
     proc $this {args} " eval notice_call $this \$args "
     initObj $this Channel Notice
     set lchan [string tolower $chan]
-    set net $current(net)
     upvar #0 $this ndata NTO$net NTO
-    if [string compare $lchan *default*] { [User :: make $chan] join $this }
+    switch *default* $lchan {} default {
+	[User :: make $net $chan] join $this
+    }
     if [catch {set def $defNotice($net)}] {
-	set def $defChan($net)
+	global defaultNet defChan
+	if [catch {set def $defNotice($defaultNet)}] {
+	    set def $defChan($defaultNet)
+	}
 	set b 0
 	set d 0
     } {
@@ -59,20 +59,32 @@ proc makeNotice {chan} {
     return $this
 }
 #
+proc notice_onShow {this} { }
+#
+proc notice_setTitles {this} {
+    set nam [$this name]
+    set id {}
+    switch nil [set usr [User :: find $nam [$this net]]] {} default {
+	switch {} [set id [$usr id]] {} default { set id " ($id)"}
+    }
+    return [list $nam "Notice from $nam$id"]
+}
+#
 proc notice_nickChange {this usr nnk} {
-    if ![string compare [$this lname] [$usr lname]] { $this nChange $nnk }
+    switch -- [$this lname] [$usr lname] {$this nChange $nnk}
     channel_nickChange $this $usr $nnk
 }
 #
 proc notice_nChange {this nnk} {
+    set net [$this net]
     if [$this active] {
-	global Icon
-	set w [$this window]
-	wm title $w "IRC Notice from $nnk"
-	wm iconname $w [set Icon($w) $nnk]
+	[$this wid] setIcon $this $nnk
+	switch nil [set usr [User :: find [$this name] $net]] {} default {
+	    switch {} [set id [$usr id]] {} default { set id " ($id)"}
+	}
+	[$this wid] setTitle $this "Notice from $nnk$id"
     }
-    upvar #0 NTO[$this net] NTO
-    upvar #0 $this cdata
+    upvar #0 NTO$net NTO $this cdata
     set ln [string tolower $nnk]
     unset NTO($cdata(lname))
     set NTO($ln) $this
@@ -87,39 +99,32 @@ proc notice_replace {this usr1 usr2} {
 #
 proc notice_delete {this} { mcnDelete $this NTO[$this net] notices }
 #
-proc Notice_make {nk} {
-    global current
-    upvar #0 NTO$current(net) NTO
-    set usr [User :: make $nk]
+proc Notice_make {net nk} {
+    upvar #0 NTO$net NTO
+    set usr [User :: make $net $nk]
     set ln [string tolower $nk]
-    if [info exists NTO($ln)] { set id $NTO($ln) } { set id [Notice $nk] }
+    if [info exists NTO($ln)] { set id $NTO($ln) } {
+	set id [$net eval [list Notice $nk]]
+    }
     $id configure -crypt [$usr crypt]
     $id show -nofocus
     $id addUser $usr 0 0
     return $id
 }
 #
-proc Notice_find {nk} {
-    global current
-    upvar #0 NTO$current(net) NTO
+proc Notice_find {nk net} {
+    upvar #0 NTO$net NTO
     set ln [string tolower $nk]
     return [expr {[info exists NTO($ln)] ? $NTO($ln) : {nil}}]
 }
 #
-proc Notice_save {desc net} {uplevel #0 defSave $desc \$defNotice($net) Notice}
+proc Notice_save {desc net} { defSave $desc  defNotice $net Notice }
 #
-proc notice_save {this desc} {
-    global defNotice
-    puts $desc [mncSave $this $defNotice([$this net])]
-    foreach  b [$this bindings] {
-	puts $desc "zbind [$this name] [lindex $b 0] {[lindex $b 1]}"
-    }
-}
+proc notice_save {this desc} { }
 #
 proc notice_leave {this} {
-    global ztrans
     set chan [$this name]
-    mkDialog LEAVE .@$this "Leaving Notice $chan" \
-      "Really close notice from $chan?" {} \
-      "$ztrans(ok) {$this delete}" "$ztrans(cancel) {}"
+    if [askUser LEAVE "Leaving Notice $chan" "Really close notice from $chan?"] {
+	$this delete
+    }
 }
