@@ -1,42 +1,27 @@
 #
 # $Source: /home/nlfm/Working/Zircon/Released/lib/RCS/User.tcl,v $
-# $Date: 1996/09/05 13:15:14 $
-# $Revision: 1.17.1.6 $
+# $Date: 1997/05/09 10:20:55 $
+# $Revision: 1.18.1.25 $
 #
-class User {
-    name	{}
-    lname	{}
-    channels	{}
-    refcount	0
-    crypt	{}
-    net		{}
-    fobj	nil
-    menu	0
-    notify	0
-    id		{}
-}
+package provide zircon 1.18
 #
 proc User {name args} {
-    if ![string compare :: $name] {
-	return [eval User_[lindex $args 0] [lrange $args 1 end]]
-    }
+    switch -- $name :: {return [eval User_[lindex $args 0] [lrange $args 1 end]]}
     set this [objName User]
     initObj $this User
     proc $this {args} "eval user_call $this \$args"
-    global current
-    set net $current(net)
-    upvar #0 UTO$net UTO
-    upvar #0 $this udata
+    upvar #0 currentNet net
+    upvar #0 UTO$net UTO $this udata
     set name [$net trimNick $name]
     set udata(name) $name
     set udata(net) $net
     $net register users $this
-    if [string compare nil [set fobj [Friend :: find $name]]] {
+    switch nil [set fobj [Friend :: find $name $net]] {} default {
 	$fobj configure -usr $this
 	set udata(fobj) $fobj
     }
     set UTO([set udata(lname) [string tolower $name]]) $this
-    if ![string match {} $args] { eval $this configure $args }
+    eval $this configure $args
     return $this
 }
 #
@@ -63,7 +48,7 @@ proc user_call {this op args} {
     upvar #0 $this udata
     switch $op {
     ref { incr udata(refcount) ; return }
-    deref { if {[incr udata(refcount) -1] <= 0} {$this delete} ; return }
+    deref { incr udata(refcount) -1 ;  return }
     }
     if [info exists udata($op)] { return $udata($op) }
     return [eval user_$op $this $args]
@@ -73,7 +58,7 @@ proc user_delete {this} {
     global TFn TFa TFg TBg TAF TAB TBl Bl IFlag OType
     upvar #0 $this udata
     [set net $udata(net)] deregister users $this
-    if [string compare nil $udata(fobj)] { $udata(fobj) configure -usr nil }
+    switch nil $udata(fobj) {} default { $udata(fobj) configure -usr nil }
     upvar #0 UTO$net UTO
     unset UTO([$this lname]) OType($this) udata
     foreach z {TFn TFa TFg TBg TAF TAB TBl Bl IFlag} {
@@ -87,7 +72,7 @@ proc user_substitute {this orig} {
     foreach x [$this channels] { $x replace $this $orig }
     set ln [$this lname]
     foreach x {Message Notice Chat} {
-	if [string compare nil [set old [$x :: find $ln]]] {
+	switch nil [set old [$x :: find $ln [$this net]]] {} default {
 	    $old replace $this $orig
 	}
     }
@@ -104,9 +89,7 @@ proc user_rename {this nk} {
     upvar #0 UTO$net UTO
     set nk [$net trimNick $nk]
     upvar #0 $this udata
-    if [string compare nil [set fobj [$this fobj]]] {
-	[$net finfo] rename $fobj $nk
-    }
+    switch nil [set f [$this fobj]] {} default {[$net finfo] rename $f $nk}
     set udata(name) $nk
     unset UTO($udata(lname))
     set UTO([set udata(lname) [string tolower $nk]]) $this
@@ -135,21 +118,22 @@ proc user_leave {this chan} {
 #
 proc user_doNotify {this} {
     if [$this notify] {
-	set fobj [Friend :: make [$this name]]
+	set fobj [Friend :: make [$this net] [$this name]]
 	$fobj configure -notify 1
 	$this configure -fobj $fobj
 	[$this net] ISON
     } {
-	if [string compare nil [set fobj [$this fobj]]] {
-	    if [$fobj ison] {
-		[[$this net] finfo] mark $fobj {}
-	    }
+	switch nil [set fobj [$this fobj]] {} default {
+	    if [$fobj ison] {[[$this net] finfo] mark $fobj {}}
 	    $fobj configure -notify 0
 	}
     }
 }
 #
-proc user_off {this} {$this unChServ}
+proc user_off {this} {
+    $this unChServ
+    handleOn [$this net] ISOFF [$this name]
+}
 #
 proc user_finger {this} { finger [$this net] [$this name] }
 #
@@ -164,46 +148,6 @@ set zFileTypes {
   {{Wav Sounds} {.\[wW\]\[aA\]\[vV\]}}
 }
 #
-proc user_dcc {this cmd} {
-    global ztrans
-    set nk [$this name]
-    set net [$this net]
-    switch $cmd {
-    SEND {
-	    if [string match {7.[67]} [info tclversion]] {
-		global SendDir zFileTypes
-		if [catch {set dir $SendDir($net)}] {
-		    set SendDir($net) [set dir [pwd]]
-		}
-		DCCSend $this [tk_getOpenFile -initialdir $dir \
-		  -title "DCC send file to $nk" -filetypes $zFileTypes]
-	    } {
-		mkFileBox {} SendDir($net) .* "Send $nk" \
-		  "File to send to $nk" {} \
-		  "$ztrans(send) {DCCSend $this}" "$ztrans(cancel) {}"
-	    }
-	}
-    CHAT {
-	    upvar #0 AChat$net AChat
-	    if [info exist AChat($this)] {
-		mkDialog {} .@chat$this {Chat} \
-		  "You already have a chat request pending for $nk." {} \
-		  "$ztrans(close) {$this unChat}" "$ztrans(keep) {}"
-	    } \
-	    elseif {[string compare nil [Chat :: find $nk]]} {
-		mkDialog {} .@chat$this {Chat} \
-		  "You already have a chat session open to $nk." {} \
-		  "$ztrans(keep) {}" "$ztrans(close) {$this unChat}"
-	    } \
-	    elseif {[catch {ChatServer $this $nk} msg]}  {
-		[$this net] errmsg "[ipAddress] : $msg"
-	    } \
-	    elseif {[winfo exists .@dls$net ]} { buildDCCList $net }
-
-	}
-    }
-}
-#
 proc user_heal {this} {
     global Split Heal TSplit
     set told 0
@@ -214,24 +158,24 @@ proc user_heal {this} {
 	    if ![info exists Heal($sl)] {
 		set told 1
 		[$net info] optText HEAL "*** Heal - $sl"
-		handleOn HEAL $sl
+		handleOn $net HEAL $sl
 	    }
 	    set v $Split($sl)
 	    listdel v $x
 	    $this deref
-	    if ![string match {} $v] {
+	    switch {} $v {
+		    unset Split($sl)
+		    catch {after cancel Heal($sl) ; unset Heal($sl)}
+	    } default {
 		set Split($sl) $v
 		catch {after cancel $Heal($sl)}
 		set Heal($sl) [after 120000 $net cleanSplit "{$sl}"]
-	    } {
-		unset Split($sl)
-		catch { after cancel Heal($sl) ; unset Heal($sl) }
 	    }
 	}
     }
     foreach x {Notice Message} {
-	if {[string compare nil [set id [$x :: find [$this lname]]]] 
-	   && [$id active]} {
+	switch nil [set id [$x :: find [$this lname] $net]] continue 
+	if [$id active] {
 	    $id addText {} {*** Heal}
 	    $id flag normal
 	}
@@ -243,22 +187,6 @@ proc user_heal {this} {
     }
 }
 #
-proc user_unChServ {this} {
-    upvar #0 AChat[$this net] AChat
-    if [info exist AChat($this)] {
-	catch {uplevel #0 unset $AChat($this)}
-	catch {shutdown $AChat($this) all}
-	catch {close $AChat($this)}
-	unset AChat($this)
-	$this deref
-    }
-}
-#
-proc user_unChat {this} {
-    $this unChServ
-    if [string compare [set id [Chat :: find [$this name]]] nil] {$id delete}
-}
-#
 proc user_kill {this} { kill [$this net] [$this name] }
 #
 proc user_pack {this net} {
@@ -279,51 +207,39 @@ proc user_unpack {this net} {
 proc user_split {this split} {
     global Split
     set net [$this net]
-    if ![info exists Split($split)] {
-	$net newSplit $split
-    }
-    foreach id [$net channels] {
-	if [$id isJoined $this] {
-	    set w [$id window]
-	    if {[set x [indexHack $w.users.menu [$this name] 3]] >=0} {
-		$w.users.menu entryconfigure $x -state disabled
-	    }
-	    $w.cFrm.uFrm.userBtn.$this conf -state disabled
-	} elseif {[$id monitor]} {
-	     if [winfo exists .@mon$id.users.userList.$this] {
-		.@mon$id.users.userList.$this configure -state disabled
-	     }
-	}
-    }
+    if ![info exists Split($split)] { $net newSplit $split }
+    foreach id [$net channels] {$id userSplit $this $split}
     foreach x {Notice Message} {
-	if {[string compare nil [set id [$x :: find [$this lname]]]] &&
-	  [$id active]} {
+	switch nil [set id [$x :: find [$this lname] $net]] continue
+	if [$id active] {
 	    $id addText {} "*** Netsplit - $split"
 	    $id flag disabled
 	}
     }
-    if [string compare nil [set fobj [$this fobj]]] {
+    switch nil [set fobj [$this fobj]] {} default {
 	[$net finfo] disable $fobj
 	$fobj configure -limbo 1
     }
     lappend Split($split) $this
     $this ref
-    handleOn USPLIT [list $split [$this lname]]
+    handleOn $net USPLIT [list $split [$this lname]]
 }
 #
 # Procs for the class User
 #
 #
-proc User_find {nk} {
-    global current
-    upvar #0 UTO$current(net) UTO
-    set name [string tolower [$current(net) trimNick $nk]]
+proc User_find {nk net} {
+    upvar #0 UTO$net UTO
+    set name [string tolower [$net trimNick $nk]]
     if [info exists UTO($name)] { return $UTO($name) } { return nil }
 }
 #
-proc User_make {nk} {
-    global current
-    upvar #0 UTO$current(net) UTO
-    set name [string tolower [$current(net) trimNick $nk]]
-    if [info exists UTO($name)] { return $UTO($name) } { return [User $nk] }
+proc User_make {net nk args} {
+    upvar #0 UTO$net UTO
+    set name [string tolower [$net trimNick $nk]]
+    if [info exists UTO($name)] { set nm $UTO($name) } {
+	set nm [$net eval [list User $nk]]
+   }
+   switch {} $args {} default { $nm configure -id [lindex $args 0] }
+   return $nm
 }
