@@ -1,112 +1,115 @@
 #
 # $Source: /home/nlfm/Working/Zircon/Released/lib/RCS/misc.tcl,v $
-# $Date: 1996/09/13 13:10:09 $
-# $Revision: 1.17.1.10 $
+# $Date: 1997/06/18 14:28:13 $
+# $Revision: 1.18.1.33 $
+#
+package provide zircon 1.18
 #
 proc credits {} {
-    global zircon tk_patchLevel
-    tellInfo {Zircon Credits} "The Zircon IRC Client
-Version $zircon(version) Patchlevel: $zircon(patchlevel)
-
-Brought to you by Lindsay (from an original idea by Jimbles)
-
-Sorry, there are just too many people to thank! You know who you are and I
-appreciate all your help.
-
-e-mail for problems : zircon@catless.ncl.ac.uk
-
-To join mailing list : zircon-request@newcastle.ac.uk
-
-Web Page : http://catless.ncl.ac.uk/Programs/Zircon
-
-tcl Version [info patchlevel]
-tk Version $tk_patchLevel
-[version]"
+    global zircon tk_patchLevel tcl_platform
+    set h [winfo screenheight .]
+    set w [winfo screenwidth .]
+    set iw [image width advert]
+    set ih [image height advert]
+    toplevel .@abt
+    wm overrideredirect .@abt 1
+    wm resizable .@abt 0 0
+    grid [frame .@abt.f -borderwidth 4 -relief raised -bg white]
+    label .@abt.f.ad -relief raised -borderwidth 2 -image advert
+    grid .@abt.f.ad
+    grid [label .@abt.f.i1 -text "Patchlevel: $zircon(patchlevel)" \
+      -fg blue -bg white]
+    grid [label .@abt.f.i2 -text "Tcl: [info patchlevel] Tk: $tk_patchLevel" \
+       -fg blue -bg white]
+    bind .@abt.f.i1 <1> { destroy .@abt }
+    bind .@abt.f.i2 <1> { destroy .@abt }
+    bind .@abt.f.ad <1> { destroy .@abt }
+    bind .@abt.f <1> { destroy .@abt }
+    wm geometry .@abt +[expr ($w -[winfo reqwidth .@abt]) /2]+[expr ($h - [winfo reqheight .@abt])/2]
 }
 #
 proc viewMode {net chan args} { $net qSend MODE :$chan }
 #
 proc doLimit {net chan string} {
-    if [string compare $string 0] {$net MODE $chan +l $string} {unlimit $net $chan}
+    switch 0 $string {unlimit $net $chan} default {$net MODE $chan +l $string}
 }
 #
 proc unlimit {net chan args} { $net MODE ${chan} -l }
 #
 proc channel_setLimit {this} {
-    global ztrans
     set chan [$this name]
     set net [$this net]
     if [$this operator] {
-    mkEntryBox .@l$this $ztrans(limit) \
-      "Enter limit value for $chan:" "{$ztrans(limit) {}}" \
-      "$ztrans(set) {doLimit $net {$chan}}" \
-      "$ztrans(view) {viewMode $net {$chan}}" \
-      "$ztrans(clear) {unlimit $net {$chan}}" "$ztrans(cancel) {}"
+    mkEntryBox .@l$this [trans limit] "Enter limit value for $chan:" \
+      [list [list limit {}]] \
+      [list set "doLimit $net $chan"] \
+      [list view "viewMode $net $chan"] \
+      [list clear "unlimit $net $chan"] \
+      [list cancel {}]
     } {
 	viewMode $net $chan
     }
 }
 #
+proc reallyKick {net chan who msg} {$net KICK $chan $who $msg}
+#
 proc channel_kick {this usr} {
-    global ztrans
     set chan [$this name]
     set who [$usr name]
-    mkDialog {} .@k$this $ztrans(kick) "Really kick $who from channel $chan?" \
-      "{$ztrans(message) {}}" "$ztrans(ok) {[$this net] KICK {$chan} {$who}}" \
-      "$ztrans(cancel) {}"
+    mkDialog {} .@k$this [trans kick] "Really kick $who from channel $chan?" \
+      [list [list message {}]] \
+      [list ok "reallyKick [$this net] [list $chan] [list $who]"] \
+      [list cancel {}]
 }
 #
 proc channel_banKick {this usr} {
     global banInfo
     set banInfo([$this net]) [list $usr $this]
+    $usr ref
     [$this net] qSend USERHOST :[$usr name]
 }
 #
 proc channel_banList {this args} { [$this net] qSend MODE [$this name] :+b }
 #
-proc doBan {net op chid string} { 
-    if ![string match {} $string] {
-	$net qSend MODE [$chid name] ${op}b :$string
-    }
+proc doBan {net op chid string} {
+    switch {} $string return
+    $net qSend MODE [$chid name] ${op}b :$string
 }
 #
 proc channel_setBan {this} {
-    global ztrans
     set net [$this net]
     if [$this operator] {
-	mkEntryBox .@ban$this $ztrans(ban) \
+	mkEntryBox .@ban$this [trans ban] \
 	  "Enter name to be banned/unbanned from [$this name]." \
-	  "{$ztrans(pattern) {}}" "$ztrans(ban) {doBan $net + $this}" \
-	  "$ztrans(unban) {doBan $net - $this}" \
-	  "$ztrans(list) {$this banList}" "$ztrans(cancel) {}"
+	  [list [list pattern {}]] [list ban "doBan $net + $this"] \
+	  [list unban "doBan $net - $this"] \
+	  [list list "$this banList"] [list cancel {}]
     } {
 	$this banList
     }
 }
 #
 proc doKey {chid string} {
-    if [string match {} $string] {
-	clearKey ${chid}
-    } {
-	global ztrans
-	mkDialog SETKEY {} {Set Key} \
-	  "Really set key for channel [$chid name]?" {} \
-	  "$ztrans(ok) {doSetKey $chid {$string}}" "$ztrans(cancel) {}"
+    switch {} $string {
+	clearKey $chid
+    } default {
+	if [askUser SETKEY {Set Key} "Really set key for channel [$chid name]?"] {
+	    doSetKey $chid $string
+	}
     }
 }
 #
 proc doSetKey {chid string} {
-    if ![string match {} [$chid key]] { doClearKey $chid }
+    switch {} [$chid key] {} default {doClearKey $chid}
     $chid configure -key $string
     [$chid net] MODE [$chid name] +k $string
 }
 #
 proc clearKey {chid args} {
-    if [string match {} [$chid key]] return
-    global ztrans
-    mkDialog CLEARKEY {} {Clear Key} \
-      "Really clear key for channel [$chid name]?" {} \
-      "$ztrans(ok) {doClearKey $chid}" "$ztrans(cancel) {}"
+    switch {} [$chid key] return
+    if [askUser CLEARKEY {Clear Key} "Really clear key for channel [$chid name]?"] {
+	doClearKey $chid
+    }
 }
 #
 proc doClearKey {chid args} {
@@ -115,24 +118,22 @@ proc doClearKey {chid args} {
 }
 #
 proc channel_setKey {this} {
-    global ztrans
     set ch [$this name]
     set net [$this net]
     if [$this operator] {
-	mkEntryBox .@k$this $ztrans(key) "Enter key for $ch:" \
-	  "{$ztrans(key) [$this key]}" "$ztrans(set) {doKey $this}" \
-	  "$ztrans(view) {viewMode $net {$ch}}" \
-	  "$ztrans(clear) {clearKey $this}" "$ztrans(cancel) {}"
+	mkEntryBox .@k$this [trans key] "Enter key for $ch:" \
+	  [list [list key [$this key]]] [list set "doKey $this"] \
+	  [list view "viewMode $net [list $ch]"] \
+	  [list clear "clearKey $this"] [list cancel {}]
     } {
 	viewMode $net $ch
     }
 }
 #
 proc finger {net nk} {
-    if ![string match {} $nk] {
-	global fingerInfo
-	$net qSend USERHOST [set fingerInfo($net) :[$net trimNick [cleanup $nk]]]
-    }
+    switch {} $nk return
+    global fingerInfo
+    $net qSend USERHOST [set fingerInfo($net) :[$net trimNick [cleanup $nk]]]
 }
 #
 proc doBanKick {net who chan msg ptr} {
@@ -141,45 +142,51 @@ proc doBanKick {net who chan msg ptr} {
 }
 #
 proc irc302 {net prefix param pargs} {
-    global banInfo ignoreInfo fingerInfo signInfo ztrans
-    if ![regexp {^(.*)(\*?)=([+-])(.*)$} $param match nk op away uh] {
-	if [info exists fingerInfo($net)] { set nk $fingerInfo($net) } \
-	elseif {[info exists banInfo($net)]} { set nk $banInfo($net) } \
-	elseif {[info exists ignoreInfo($net)]} { set nk $ignoreInfo($net)} { set nk {} }
-	catch {unset banInfo($net) ignoreInfo($net) fingerInfo($net)}
-	tellError Nickerr "No such nick as $nk!"
+    global banInfo fingerInfo signInfo
+    if ![regexp {^([^*]*)(\*?)=([+-])([~+=^-]?)(.*)$} $param match nk op away ident uh] {
+	if [info exists fingerInfo($net)] {
+	    set nk $fingerInfo($net)
+	    unset fingerInfo($net)
+	} \
+	elseif {[info exists banInfo($net)]} {
+	    set usr [lindex $banInfo($net) 0]
+	    set nk [$usr name]
+	    $usr deref
+	    unset banInfo($nrt)
+	} { set nk {} }
+	tellError $net Nickerr "No such nick as $nk!"
 	return
     }
-    set usr [User :: find $nk]
-    set frd [Friend :: find  $nk]
+    set usr [User :: make $net $nk]
+    set frd [Friend :: find  $nk $net]
     if {[info exists banInfo($net)] && [lindex $banInfo($net) 0] == $usr} {
 	set chan [[lindex $banInfo($net) 1] name]
 	set who [$usr name]
-	mkEntryBox {} "Ban+Kick" \
-	  "Really ban and kick $who ($uh) from channel $chan?" \
-	  [list "$ztrans(message) {}" [list $ztrans(pattern) "*!*$uh"]] \
-	  "$ztrans(ok) {doBanKick $net {$who} {$chan}}" "$ztrans(cancel) {}"
+	mkEntryBox {} Ban+Kick \
+	  "Really ban and kick $who ($ident$uh) from channel $chan?" \
+	  [list [list message {}] [list pattern *!*$uh]] \
+	  [list ok "doBanKick $net [list $who] [list $chan]"] [list cancel {}]
 	unset banInfo($net)
-    } \
-    elseif {[info exists ignoreInfo($net)]} {
-	unset ignoreInfo($net)
+	$usr deref
     } \
     elseif {[info exists signInfo($net)] && [set x [lsearch $signInfo($net) $frd]] >= 0} {
-	if [string match [$frd id] $uh] {
-	    global signOns
-	    append signOns($net) "$nk ($uh) "
+	global signOns signNOns
+	switch -regexp -- $uh [$frd id] {
+	    append signOns($net) "$nk ($ident$uh) "
+	    lappend signNOns($net) $nk
 	    set frnd [$net finfo]
 	    if {[$net friendsOn] && [$frd menu]} { $frnd add $frd }
 	    $frnd mark $frd ison
 	}
         listdel signInfo($net) $x
-	if [string match {} $signInfo($net)] {
+	switch {} $signInfo($net) {
 	    if [info exists signOns($net)] {
-		mkInfoBox ISON {} Notify \
-		  "[getDate] :\nSignon by $signOns($net)" {Dismiss {}} \
-		  "WHOIS {who303 $net $signOns($net)}"
+		mkInfoBox $net ISON {} Notify \
+		  "[getDate] :\nSignon by $signOns($net)" {dismiss {}} \
+		  [list whois "who303 $net [list $signNOns($net)]"]
+		foreach x $signNOns($net) { handleOn $net ISON [list $x] }
 	    }
-	    catch {unset signInfo($net) signOns($net)}
+	    catch {unset signInfo($net) signOns($net) signNOns($net)}
 	}
     } \
     elseif {[info exists fingerInfo($net)]} {
@@ -194,24 +201,24 @@ proc irc302 {net prefix param pargs} {
 		    destroy $w
 		    catch {close $sock}
 		"
-
-		set oft [frame $w.oFrm]
-		scrollbar $oft.vscroller -command "$oft.text yview"
-		text $oft.text -yscrollcommand "$oft.vscroller set"
-		pack $oft.text -side left -fill both -expand 1
-		pack $oft.vscroller -side right -fill y
-		button $w.ok -text Dismiss -command "
+		grid columnconfigure $w 0 -weight 1
+		grid rowconfigure $w 0 -weight 1
+		scrollbar $w.vscroller -command "$w.text yview"
+		text $w.text -yscrollcommand "$w.vscroller set"
+		grid $w.text -row 0 -column 0 -sticky nsew
+		grid $w.vscroller -row 0 -column 1 -sticky ns
+		grid [button $w.ok - -text Dismiss -command "
 		    destroy $w
 		    catch {close $sock}
-		"
-		pack $w.oFrm -expand 1 -fill x
-		pack $w.ok -fill x
+		"] -
 		puts $sock $user@$host
 	} {
 	    $net errmsg "Finger Error $uh : $sock"
 	}
     } {
-	$net inform "$nk is $uh (${op}${away})"
+	switch {} $op {} default {set op "(IRC Operator) "}
+	switch -glob -- $away *+* {set away "Not Away"} default {set away Away}
+	$net inform "$nk is $ident$uh ${op}($away)"
     }
 }
 #
@@ -221,7 +228,7 @@ proc handleFinger {net conn w} {
     } \
     elseif {[winfo exists $w]} {
 	regsub -all "\r" $msg {} msg
-	$w.oFrm.text insert end $msg\n
+	$w.text insert end $msg\n
     }
 }
 #
@@ -241,12 +248,21 @@ proc irc312 {net prefix param pargs} {
 #
 proc irc313 {net prefix param pargs} { global whois ; set whois($net,ircop) 1 }
 #
+proc whoText {net} {
+    global whois
+    set txt "Name: $whois($net,info1)@$whois($net,info2)\
+ ($whois($net,info3))\nServer: $whois($net,info4) ($whois($net,info5))"
+    if [info exists whois($net,away)] {
+	append txt "\nAway: $whois($net,away)"
+    }
+    return $txt
+}
+#
 proc irc314 {net prefix param pargs} {
     global whois whowas
     if [info exists whois($net,info0)] {
 	if [info exists whowas($net)] { append whowas($net) "\n\n" }
-	append whowas($net) "Name: $whois($net,info1)@$whois($net,info2)\
-	  ($whois($net,info3))\nServer: $whois($net,info4) ($whois($net,info5))"
+	append whowas($net) [whoText $net]
 	foreach x [array names whois $net,*] { unset whois($x) }
     }
     irc311 $net $prefix $param $pargs
@@ -255,9 +271,7 @@ proc irc314 {net prefix param pargs} {
 proc irc317 {net prefix param pargs} {
     global whois
     set val [lindex $pargs 2]
-    if {$val == 1} {
-	set whois($net,time) "1 second"
-    } {
+    switch 1 $val {set whois($net,time) "1 second"} default {
 	if {$val >= 60} {
 	    if {$val < 120} {
 		set whois($net,time) "1 minute"
@@ -283,39 +297,41 @@ proc irc318 {net prefix param pargs} {
     if [info exists whois($net,time)] { append txt "Idle: $whois($net,time)\n" }
     if [info exists whois($net,ircop)] { append txt "$who is an IRC operator.\n" }
     if [info exists whois($net,away)] {
-	set wd [max $wd [string length $whois(away)]]
+	set wd [max $wd [string length $whois($net,away)]]
 	append txt "Away: $whois($net,away)\n"
     }
-    set w .@[newName ws]
+    set w .@[newName whos]
     catch {destroy $w}
     toplevel $w -class Zircon
     wm title $w "WHOIS $who"
-    wm protocol $w WM_DELETE_WINDOW "destroy $w"
-    frame $w.f1 -borderwidth 0
-    text $w.f1.t -relief raised -height 5 -width $wd
-    $w.f1.t insert end $txt
-    frame $w.f1.b -relief raised
-    pack $w.f1.b -fill x -side bottom
-    pack $w.f1.t -expand 1 -fill both -side top
-    button $w.f1.b.ok -text Dismiss -command "destroy $w"
-    button $w.f1.b.msg -text Message -command "doMsg $net {$who}"
-    pack $w.f1.b.ok $w.f1.b.msg -expand 1 -side left -fill x
-    pack $w.f1 -fill both -expand 1 -side left
+    grid [text $w.t -relief raised -height 5 -width $wd] -sticky nsew
+    grid columnconfigure $w 0 -weight 1
+    grid rowconfigure $w 0 -weight 1
+    $w.t insert end $txt
+    $w.t configure -state disabled
+    grid $w.t -sticky nsew
+    grid [frame $w.b -relief raised] - -sticky ew
+    grid columnconfigure $w.b 0 -weight 1
+    grid columnconfigure $w.b 1 -weight 1
+    button $w.b.ok -text Dismiss -command "destroy $w"
+    button $w.b.msg -text Message -command "doMsg $net {$who}"
+    grid $w.b.ok $w.b.msg -sticky ew
     if {[info exists whois($net,channels)] && \
       ![string match {} $whois($net,channels)]} {
-	button $w.f1.b.all -text {Join All} \
+	button $w.b.all -text {Join All} \
 	  -command "joinAll $net {$whois($net,channels)} ; destroy $w"
-	pack $w.f1.b.all -expand 1 -side left -fill x
+	grid $w.b.all -row 0 -column 2 -sticky ew
+	grid columnconfigure $w.b 2 -weight 1
 	makeLB $w.f2
 	foreach chn $whois($net,channels) { $w.f2.l insert end $chn }
 	bind $w.f2.l <Double-Button-1> "joinAll $net \[%W get \[%W nearest %y\]\]"
-	pack $w.f2 -side right -fill both -expand 1
+	grid $w.f2 -row 0 -column 1 -sticky nsew
     }
     foreach x [array names whois $net,*] { unset whois($x) }
 }
 #
 proc joinAll {net arg} {
-    foreach ch $arg { regsub {^@} $ch {} ch ; channelJoin $net $ch }
+    foreach ch $arg { regsub {^[@+]} $ch {} ch ; channelJoin $net $ch }
 }
 #
 proc irc319 {net prefix param pargs} {
@@ -325,19 +341,23 @@ proc irc319 {net prefix param pargs} {
 proc irc369 {net prefix param pargs} {
     global whois whowas
     if [info exists whois($net,err)] {
-	set txt "There was no such user as $whois($net,err)."
+	set txt "There was no such user as [set who $whois($net,err)]."
     } {
-	if [info exists whowas($net)] { append whowas($net) "\n\n" } { set whowas($net) {} }
-	set txt "$whowas($net)Name: $whois($net,info1)@$whois($net,info2) ($whois($net,info3))\n\
-Server: $whois($net,info4) ($whois($net,info5))"
+	if [info exists whowas($net)] {
+	    set txt $whowas($net)\n\n
+	} {
+	    set txt {}
+	}
+	append txt [whoText $net]
+	set who $whois($net,info0)
     }
-    mkInfoBox WHOWAS .@whowas Whowas "$txt" {Dismiss {}}
+    mkInfoBox $net WHOWAS .@[newName whow] "WHOWAS $who" $txt "dismiss {}"
     foreach x [array names whois $net,*] { unset whois($x) }
     catch {unset whowas($net)}
 }
 #
 proc irc341 {net prefix param pargs} {
-    if ![string compare nil [set id [Channel :: find [set chan [lindex $pargs 2]]]]] {
+    switch nil [set id [Channel :: find [set chan [lindex $pargs 2]] $net]] {
 	set id [$net info]
     }
     $id addText {} "*** Inviting [lindex $pargs 1] to channel ${chan}"
@@ -361,7 +381,7 @@ proc irc352 {net prefix param pargs} {
       [lindex $pargs 5] [lindex $pargs 6] [lindex $pargs 2] \
       [lindex $pargs 3] $param]
     if ![info exists whoTxt($net)] {
-	set whoTxt($net) [mkInfoBox WHO {} "Who [getDate]" {} {Dismiss {}}]
+	set whoTxt($net) [mkInfoBox $net WHO {} "Who [getDate]" {} "dismiss {}"]
 	$whoTxt($net) configure -tabs {1i 2i 3i}
     }
     if ![winfo exists $whoTxt($net)] return
@@ -370,7 +390,7 @@ proc irc352 {net prefix param pargs} {
     $whoTxt($net) configure -state disabled
     set ln [lindex [split [$whoTxt($net) index end] .] 0]
     if {$ln < 24 && $ln > 10} {
-	$whoTxt($net) conf -height $ln
+	$whoTxt($net) configure -height $ln
     }
     $whoTxt($net) see end
 }
@@ -378,7 +398,7 @@ proc irc352 {net prefix param pargs} {
 proc irc367 {net prefix param pargs} {
     set chan [lindex $pargs 1]
     set ban [lindex $pargs 2]
-    if {[string compare nil [set chn [Channel :: find $chan]]] && [$chn active]} {
+    if {[string compare nil [set chn [Channel :: find $chan $net]]] && [$chn active]} {
 	$chn addText @BAN "**> $ban is banned."
     } {
 	$net display @BAN "Channel $chan bans $ban"
@@ -387,7 +407,7 @@ proc irc367 {net prefix param pargs} {
 #
 proc irc368 {net prefix param pargs} {
     set chan [lindex $pargs 1]
-    if {[string compare nil [set chn [Channel :: find $chan]]] && [$chn active]} {
+    if {[string compare nil [set chn [Channel :: find $chan $net]]] && [$chn active]} {
 	$chn addText @BAN "**> $param"
     } {
 	$net display @BAN "Channel $chan $param"
@@ -397,7 +417,7 @@ proc irc368 {net prefix param pargs} {
 proc handleURL {net url} {
     global zircon
     if [info exists zircon(cciport)] {
-    if ![string compare $zircon(cciport) netscape] {
+	if ![string compare $zircon(cciport) netscape] {
 	    if [catch {exec netscape -remote openurl($url,newwindow)}] {
 		exec netscape $url &
 	    }
@@ -418,37 +438,32 @@ proc handleURL {net url} {
 }
 
 #
-proc doExec {where} {
-    global ztrans
+proc doExec {net where} {
     mkDialog EXEC .@e$where {Execute command} {Enter command to be executed} \
-	{{Command {}}} "$ztrans(ok) {runCmd $where}" "$ztrans(cancel) {}"
+	{{Command {}}} "ok {runCmd $net $where}" "cancel {}"
 }
 #
-proc runCmd {where cmd} {
-    if ![string match {} $cmd] {
-	if [catch {open "|$cmd 2>&1" r} ip] {
-	    tellError {Execerr} "Error executing \"$cmd\" - $ip"
-	} {
-	    fileevent $ip readable "execOP $where {$cmd} $ip"
-	}
+proc runCmd {net where cmd} {
+    switch {} $cmd return
+    if [catch {open "|$cmd 2>&1" r} ip] {
+	tellError $net Execerr "Error executing \"$cmd\" - $ip"
+    } {
+	fileevent $ip readable "execOP $net $where {$cmd} $ip"
     }
 }
 #
-proc net_exec {this} { doExec [$this info] }
+proc net_exec {this} { doExec $this [$this info] }
 #
-proc execOP {where cmd fd} {
+proc execOP {net where cmd fd} {
     if [catch {gets $fd} data] {
-	tellError {Execerr} "Error executing \"$cmd\" - $data"
+	tellError $net Execerr "Error executing \"$cmd\" - $data"
     } \
     elseif {[eof $fd]} {
 	if [catch {close $fd} msg] {
-	    tellError Execerr "Error executing \"$cmd\" - $msg"
+	    tellError $net Execerr "Error executing \"$cmd\" - $msg"
 	}
-    } \
-    elseif {[string match {info*} $where]} {
-	$where addText EXEC $data
     } {
-	$where send $data
+       switch $where info* {$where addText EXEC $data} default {$where send $data}
     }
 }
 #
@@ -468,7 +483,17 @@ proc findURL {win x y net} {
     }
     notIdle %W
 }
-
-
-
-
+#
+proc doHelp {net topic args} {
+    switch {} [set s [lindex $args 0]] {} default {$net msg $s $topic}
+}
+#
+proc getHelp {net} {
+    switch {} [$net helpService] {
+	set ents [list [list topic {zircon ?}] [list service [$net helpService]]]
+    } default {
+	set ents [list [list topic {zircon ?}]]
+    }
+    mkEntryBox .@help$net [trans help] {Enter topic on which you need help:} \
+      $ents [list ok "doHelp $net"] [list cancel {}]
+}
