@@ -1,16 +1,18 @@
 #
 # $Source: /home/nlfm/Working/Zircon/Released/lib/RCS/misc2.tcl,v $
-# $Date: 1996/06/20 19:56:05 $
-# $Revision: 1.17.1.2 $
+# $Date: 1997/05/09 10:21:13 $
+# $Revision: 1.18.1.17 $
 #
 #
+package provide zircon 1.18
+#
 proc net_doMisc2 {this chid win} {
     set line [$win get]
     $win delete 0 end
     if [regexp {^(/[a-zA-Z]+)( (.*))?$} $line dummy pr r1 rest] {
 	set pr [string toupper $pr]
-	if ![string match {} [info procs $pr]] {
-	    $chid addText {} "!> $line"
+	switch {} [info procs $pr] {} default {
+	    $chid addText @me "!> $line"
 	    $chid configure -hpos end
 	    $pr $this $chid $rest
 	    return
@@ -19,6 +21,8 @@ proc net_doMisc2 {this chid win} {
     $chid send $line
 }
 #
+proc /ABORT {net chid arg} { exit }
+#
 proc /ADMIN {net chid arg} { $net nsend ADMIN [lindex $arg 0]}
 #
 proc /ALIAS {net chid arg} { sillyPerson $net }
@@ -27,11 +31,19 @@ proc /ASSIGN {net chid arg} { sillyPerson $net }
 #
 proc /AWAY {net chid arg} { $net AWAY $arg }
 #
+proc /BEEP {args} { bell }
+#
 proc /BIND {net chid arg} { sillyPerson $net }
 #
 proc /BYE {net chid arg} { $net doQuit [lindex $arg 0]}
 #
-proc /CD {net chid arg} { sillyPerson $net }
+proc /CD {net chid arg} {
+    if [string match {} $arg] {
+	tellInfo {Working Directory} [pwd]
+    } {
+	cd $arg
+    }
+}
 #
 proc /CHANNEL {net chid arg} { channelJoin $net [lindex $arg 0] }
 #
@@ -45,7 +57,7 @@ proc /CONNECT {net chid arg} {
 #
 proc /CTCP {net chid arg} {
     set nk [lindex $arg 0]
-    if ![string compare $nk *] { set nk [$chid name] }
+    switch * $nk {set nk [$chid name]}
     switch [set cmd [string toupper [lindex $arg 1]]] {
     PING { $net CTCP PING $nk [ZPING] }
     CLIENTINFO {$net CTCP $CMD $nk [lindex $arg 2]}
@@ -61,17 +73,59 @@ proc /CTCP {net chid arg} {
 #
 proc /DATE {net chid arg} { $net TIME $arg }
 #
-proc /DCC {net chid arg} { sillyPerson $net }
+proc /DCC {net chid arg} {
+    set op [string toupper [lindex $arg 0]]
+    switch -- $op {
+    SEND {
+	   DCCSend [User :: make $net [lindex $arg 1]] [lindex $arg 2]
+	}
+    CHAT {[User :: make $net [lindex $arg 1]] dcc CHAT}
+    GET {
+	    global DCCList
+	    set gusr [User :: make $net [lindex $arg 1]]
+	    set file [lindex $arg 2]
+	    set i 0
+	    foreach x $DCCList($net) {
+		switch [set op [lindex $l 0]] Send continue
+		set usr [lindex $l 1]
+		set addr [lindex $l 2]
+		set port [lindex $l 3]
+		set fln [lindex $l 4]
+		set leng [lindex $l 5]
+		set posn [lindex $l 6]
+		switch -- $gusr $usr {
+		    switch -- $file $fln {
+		        doGetDCC $net $op $usr $addr $port $leng $posn $file
+			$usr deref
+			listdel DCCList($net) $i
+			switch {} $DCCList($net) {destroy .@drq$net}
+			return
+		    }
+		}
+		incr i
+	    }
+	    tellError {} {DCC Get} {No such send outstanding}
+	}
+    CLOSE -
+    LIST {buildDCCList $net}
+    close {
+	    set typ [lindex $arg 1]
+	    set gusr [User :: make $net [lindex $arg 2]]
+	    sillyPerson $net
+	}
+    default { sillyPerson $net }
+    }
+}
 #
 proc /DEOP {net chid arg} { $net deIRCOp }
 #
 proc /DESCRIBE {net chid arg} {
     set chan [lindex $arg 0]
-    if {[string compare nil [set ch [Channel :: find $chan]]] ||
-	[string cpmpare nil [set ch [Message :: find $chan]]]} {
+    if {[string compare nil [set ch [Channel :: find $chan $net]]] ||
+	[string cpmpare nil [set ch [Message :: find $chan $net]]]} {
 	$ch action [lrange $arg 1 end]
     } {
-	$net PRIVMSG $chan "\001ACTION [lrange $arg 1 end]\001"
+	$net msg $chan "\001ACTION [lrange $arg 1 end]\001"
     }
 }
 #
@@ -79,7 +133,11 @@ proc /DIE {net chid arg} { sillyPerson $net }
 #
 proc /DIGRAPH {net chid arg} { sillyPerson $net }
 #
-proc /DMSG {net chid arg} { sillyPerson $net }
+proc /DMSG {net chid arg} {
+    switch nil [set cht [Chat :: find [lindex $arg 0] $net]] {
+	tellError $net Error {No such DCC chat  connection}
+    } default {$cht send [lrange $arg 1 end]}
+}
 #
 proc /DQUERY {net chid arg} { sillyPerson $net }
 #
@@ -88,16 +146,16 @@ proc /ECHO {net chid arg} { $net inform $arg }
 proc /ENCRYPT {net chid arg} {
     set nk [lindex $arg 0]
     if [string match {[#&]} $nk] {
-	set chid [Channel :: make $nk]
+	set chid [Channel :: make $net $nk]
     } {
-	set chid [Message :: make $nk]
+	set chid [Message :: make $net $nk]
     }
     $chid configure -key [lindex $arg 1]
 }
 #
 proc /EVAL {net chid arg} { sillyPerson $net }
 #
-proc /EXEC {net chid arg} { runCmd $chid $arg }
+proc /EXEC {net chid arg} { runCmd $net $chid $arg }
 #
 proc /EXIT {net chid arg} { $net doQuit [lindex $arg 0]}
 #
@@ -127,13 +185,17 @@ proc /INVITE {net chid arg} {
 #
 proc /JOIN {net chid arg} { channelJoin $net $arg }
 #
-proc /KICK {net chid arg} { $net KICK [lindex $arg 0] [lrange $arg 1 end] }
+proc /KICK {net chid arg} {
+    set chn [lindex $arg 0]
+    switch * $chn {set chn [$chid name]}
+    $net KICK $chn [lindex $arg 1] [lrange $arg 2 end]
+}
 #
 proc /KILL {net chid arg} {$net KILL [lindex $arg 0] [lrange $arg 1 end]}
 #
 proc /LASTLOG {net chid arg} { sillyPerson $net }
 #
-proc /LEAVE {net chid arg} { /PART $chid $arg }
+proc /LEAVE {net chid arg} { /PART $net $chid $arg }
 #
 proc /LINKS {net chid arg} { $net LINKS [lindex $arg 0] [lindex $arg 1] }
 #
@@ -147,12 +209,17 @@ proc /ME {net chid arg} { $chid action $arg }
 #
 proc /MLOAD {net chid arg} { sillyPerson $net }
 #
-proc /MODE {net chid arg} { eval $net send MODE $arg }
+proc /MODE {net chid arg} {
+    set chn [lindex [set arg [safeClean $arg]] 0]
+    switch * $chn {set chn [$chid name]}
+    eval $net send MODE \{$chn\} [lrange $arg 1 end]
+}
 #
 proc /MOTD {net chid arg} { $net send MOTD $arg }
 #
 proc /MSG {net chid arg} {
-    $net PRIVMSG [lindex $arg 0] [join [lrange $arg 1 end]]
+    set arg [safeClean $arg]
+    $net msg [lindex $arg 0] [lrange $arg 1 end]
 }
 #
 proc /NAMES {net chid arg} { $net NAMES $arg }
@@ -162,7 +229,7 @@ proc /NICK {net chid arg} { $net NICK $arg }
 proc /NOTE {net chid arg} { sillyPerson $net }
 #
 proc /NOTICE {net chid arg} {
-    $net NOTICE [lindex $arg 0] [join [lrange $arg 1 end]]
+    $net NOTICE [lindex $arg 0] [lrange $arg 1 end]
 }
 #
 proc /NOTIFY {net chid arg} { sillyPerson $net }
@@ -174,26 +241,29 @@ proc /OPER {net chid arg} {$net OPER [lindex $arg 0] [lindex $arg 1]}
 proc /PARSEKEY {net chid arg} { sillyPerson $net }
 #
 proc /PART {net chid arg} {
-    if [string match {} $arg] { $chid leave } {
-	if [string compare nil [set id [find $arg]]] { $id leave }
+    if {[string match {} $arg] || ![string compare * $arg]} {
+	$chid doLeave {}
+    } {
+	switch nil [set id [find [lindex $arg 0]]] {} default {
+	    $id doLeave [lindex $arg 1]
+	}
     }
 }
 #
 proc /PING {net chid arg} { foreach nk $arg { doCtcp $net PING $nk } }
 #
 proc /QUERY {net chid arg} {
-    set usr [User :: make $arg]
-    set msg [Message :: make [$usr name]]
+    set usr [User :: make $net $arg]
+    set msg [Message :: make $net [$usr name]]
     $msg show
     if ![$msg isJoined $usr] { $msg addUser $usr 0 0 }
 }
 #
 proc /QUIT {net chid arg} { $net doQuit $arg }
 #
-proc /QUOTE {net chid arg} {
-    regsub -all {[][$\\"{}]} $arg {\\&} cmd
-    eval $net send $cmd
-}
+proc /QUOTE {net chid arg} {  eval $net send [safeClean $arg] }
+#
+proc /RAW {net chid arg} {  eval $net send [safeClean $arg] }
 #
 proc /REDIRECT {net chid arg} { sillyPerson $net }
 #
@@ -210,7 +280,7 @@ proc /SEND {net chid arg} { $chid send $arg }
 proc /SENDLINE {net chid arg} { sillyPerson $net }
 #
 proc /SERVER {net chid arg} {
-    set sv [Server :: make [lindex $arg 0]]
+    set sv [Server :: make $net [lindex $arg 0]]
     if {[set port [lindex $arg 1]] != {}} { $sv configure -port $port }
     $net changeServer $sv
 }
@@ -258,11 +328,11 @@ proc /WHICH {net chid arg} { sillyPerson $net }
 #
 proc /WHILE {net chid arg} { sillyPerson $net }
 #
-proc /WHO {net chid arg} { eval $net send WHO $arg }
+proc /WHO {net chid arg} { eval $net send WHO [safeClean $arg] }
 #
-proc /WHOIS {net chid arg} { eval $net send WHOIS $arg }
+proc /WHOIS {net chid arg} { eval $net WHOIS [safeClean $arg] }
 #
-proc /WHOWAS {net chid arg} { eval $net send WHOWAS $arg }
+proc /WHOWAS {net chid arg} { eval $net WHOWAS [safeClean $arg] }
 #
 proc /WINDOW {net chid arg} { sillyPerson $net}
 #
@@ -272,44 +342,5 @@ proc /XTYPE {net chid arg} { sillyPerson $net}
 #
 proc sillyPerson {net} {
     bell
-    tkwait window [mkDialog {} Silly {Don't be silly! This is Zircon!!} {}]
+    tellError $net Silly {Don't be silly! This is Zircon!!}
 }
-#
-proc register {net menu} {
-    $menu delete 0 end
-    $menu add command -label {Join mailing list} -command "doZList $net"
-    $menu add command -label {Leave mailing list} -command "doZLeave $net"
-    if [file exist ~/.zirconreg] {
-	$menu add command -label {No update messages} \
-	  -command "doDeregister $net"
-    } {
-	$menu add command -label {Get update messages} \
-	  -command "doRegister $net"
-    }
-}
-#
-proc doRegister {net} {
-    global zircon
-    if ![file exists ~/.zirconreg] { 
-	doBotcom $net REGISTER "$zircon(version) PL $zircon(patchlevel)"
-	catch {close [open ~/.zirconreg w]}
-    }
-}
-#
-proc doDeregister {net} {
-    if [file exists ~/.zirconreg] { 
-	doBotcom $net DEREGISTER {}
-	exec rm -f [glob ~/.zirconreg]
-    }
-}
-#
-proc doZList {net} {
-    global ztrans
-    mkEntryBox .@lj {Join mailing List} "Enter your email address:" \
-      {{{Email address} {}}} "$ztrans(ok) {doBotcom $net ZLIST}" \
-      "$ztrans(cancel) {}"
-}
-#
-proc doZLeave {net} {doBotcom $net ZLEAVE {}}
-#
-proc doBotcom {net type par} { $net PRIVMSG ZirconBot "!$type $par" }
